package org.coderu.common.utils;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.lang3.Validate;
import org.coderu.common.utils.types.Pair;

import com.google.common.base.Function;
import com.google.common.base.Functions;


/**
 * factory for transformers 
 * @author arubinov
 *
 */
public final class FunctionFactory {
    
    /**
     * get transformer that convert an object to string using its "toString" method
     * @return transformer that convert an object to string using its "toString" method
     */
    @SuppressWarnings("unchecked")
    public static <T> Function<T,String> toStringFunction(){ // NO_UCD
        return TO_STRING;
    }

    /**
     * get first element from collection.
     * @throws exception if collection is empty
     */
    @SuppressWarnings("unchecked")
    public static <T> Function<Collection<T>, T> collection2first(){
        return COLLECTION_TO_FIRST;           
    }

    @SuppressWarnings("unchecked")
    public static <A, B> Function<A,B> cast(){
        return TO_SELF;
    }
    
    /**
     * Concatenates two transformers.
     */
    public static <A, B, C> Function<A, C> concat(
            final Function<A,B> a2b, final Function<? super B, C> b2c ){
        
        return Functions.compose(b2c,a2b);
    }

    /**
     * creates new transformer that remains source of transformation.
     * create transformer  A =&gt; Pair&lt;A,B&gt; from transformer A =&gt; B
     */
    public static <A,B> Function<A, Pair<A,B>> remainSource(final Function<A,B> transformer){
        return new Function<A, Pair<A,B>>(){
            public Pair<A,B> apply(A o) {
                return Pair.make(o,transformer.apply(o));
            };
        };
    }

    /**
     * creates a transformer that sorts a collection.
     * @throws exception if collection is empty
     */
    public static <T> Function<Collection<T>, List<T>> sorter(final Comparator<T> comp){
        
        Validate.isTrue(comp!=null, "comparater is null!");
        return new Function<Collection<T>, List<T>>(){
            public List<T> apply(Collection<T> col) {
                final List<T> result = new ArrayList<T>(col);
                Collections.sort(result,comp);
                return result;
            }
        };          
    }

    
    @SuppressWarnings("rawtypes")
    private static final Function TO_STRING =  new Function(){
        public String apply(Object o) {
            return o.toString();
        }
        @Override
        public String toString() {
            return "string";
        }
    };

    @SuppressWarnings("rawtypes")
    private static final Function TO_SELF =  new Function(){
        public Object apply(Object o) {
            return o;
        }
        @Override
        public String toString() {
            return "";
        }
    };
    @SuppressWarnings("rawtypes")
    private static final Function COLLECTION_TO_FIRST  = new Function(){
        public Object apply(Object o) {
            Collection col = (Collection)o;
            Validate.isTrue(!col.isEmpty(), "collection is empty");
            return col.iterator().next();
        }
        @Override
        public String toString() {
            return "first element";
        }
    };
    
    private FunctionFactory() {
    }
    
}
