package org.coderu.common.utils.types;

import java.io.Serializable;


/**
 * 
 * a pair of objects
 * 
 * @author Alexander Rubinov
 *
 * @param <T1> type of first object
 * @param <T2> type of second object
 */
@SuppressWarnings("serial")
public final class Pair<T1,T2> extends AbstractPair<T1, T2> implements Serializable{

    /**
     * create a pair of two object
     * @param <T1> type of first object
     * @param <T2> type of second object
     * @param first first object
     * @param second second object
     * @return a pair of two object
     */
    public static <T1,T2> Pair<T1,T2> make(T1 first, T2 second) {
        return new Pair<T1,T2>(first, second);
    }

    private Object writeReplace(){
        return new SerializationProxy(this);
    }

    private static final class SerializationProxy implements Serializable{
        private static final long serialVersionUID = 1L;
        private final Object[] data;
        
        private SerializationProxy(Pair<?,?> pair){
            this.data = new Object[]{pair.getFirst(),pair.getSecond()};
        }
        
        @SuppressWarnings({ "unchecked", "rawtypes" })
        private Object readResolve(){
            return new Pair(data[0], data[1]);
        }
    }
    private Pair(T1 first, T2 second) {
        super(first,second);
    }

    
}
