package org.coderu.common.utils.types;

import java.io.Serializable;

/**
 *
 * a triple of objects
 *
 * @author Alexander Rubinov
 *
 * @param <T1> type of first object
 * @param <T2> type of second object
 * @param <T3> type of third object
 */
public final class Triple<T1,T2,T3> extends AbstractTriple<T1,T2,T3> implements Serializable{

    private static final long serialVersionUID = 1L;

    private Triple(T1 first, T2 second, T3 third) {
        super(first,second,third);
    }

    public static <T1,T2,T3> Triple<T1,T2,T3> make(T1 first, T2 second, T3 third) {
        return new Triple<T1,T2,T3>(first, second, third);
    }
    
    private Object writeReplace(){
        return new SerializationProxy(this);
    }

    private static final class SerializationProxy implements Serializable{
        private static final long serialVersionUID = 1L;
        private final Object[] data;
        
        private SerializationProxy(Triple<?,?,?> triple){
            this.data = new Object[]{triple.getFirst(),triple.getSecond(), triple.getThird()};
        }
        
        @SuppressWarnings({ "unchecked", "rawtypes" })
        private Object readResolve(){
            return new Triple(data[0], data[1], data[2]);
        }
    }
}