package org.coderu.core.impl.explorer.parser.impl.loader;

import java.io.File;
import java.util.Collection;
import java.util.LinkedList;

import org.coderu.common.utils.types.SingleValue;


@SuppressWarnings("serial")
public final class FileCollector extends SingleValue<File>{

    public FileCollector(final File base){
        super(base);
        assert base.isDirectory();
    }

    public File getBaseDirectory(){
        return getValue();
    }

    public Collection<File> collect(){
        final Collection<File> files = new LinkedList<File>();
        collectFiles(getBaseDirectory(),files);
        return files;
    }

    private void collectFiles(File directory, Collection<File> files) {

        assert directory.isDirectory();
        
        final String[] directoryFiles = directory.list();

        for (int i = 0; i < directoryFiles.length; i++) {

            File file = new File(directory, directoryFiles[i]);
            if (acceptClassFile(file)) {
                files.add(file);
            } else if (file.isDirectory()) {
                collectFiles(file, files);
            }
        }
    }

    private static boolean acceptClassFile(File file) {
        if (!file.isFile()) {
            return false;
        }
        return isClassFile(file.getName());
    }

    public static boolean isClassFile(String name) {

        return name.toLowerCase().endsWith(".class");
    }

}
